'' =================================================================================================
''
''   File....... jm_max7219_pasm_work.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2018 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 11 FEB 2018
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                                 
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O } 


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main                                                         
                                                                 
  setup                                                          
                                                                 
  repeat                                                         
    waitcnt(0)                                                   
                                                                 
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 

con { max7219 cog }

var

  long  cog
  long  command


pub max_start(dpin, cpin, cspin)

  max_stop

  command.byte[0] := dpin
  command.byte[1] := cpin
  command.byte[2] := cspin

  cog := cognew(@entry, @command) + 1


pub max_stop

  if (cog)
    cogstop(cog-1)
    cog := 0
    

pub max_out(reg, value) : bits

  bits.byte[3] := reg                                           ' set register
  bits.byte[2] := value                                         ' set value
  bits.byte[0] := 1                                             ' force pasm

  command := bits 

  repeat while (command)
  

dat { simple pasm max7219 driver }

                        org     0

entry                   mov     t1, par                         ' get address of command
                        rdlong  t1, t1                          ' read pins
                        mov     t2, t1                          ' make a copy

                        and     t1, #$3F                        ' isolate data pin
                        mov     dmask, #1
                        shl     dmask, #1
                        andn    outa, dmask                     ' low
                        or      dira, dmask                     ' output                   

                        mov     t1, t2
                        shr     t1, #8
                        and     t1, #$3F                        ' isolate clock pin
                        mov     cmask, #1
                        shl     cmask, #1
                        andn    outa, cmask                     ' low
                        or      dira, cmask                     ' output
                        
                        mov     t1, t2
                        shr     t1, #16
                        and     t1, #$3F                        ' isolate cs pin
                        mov     csmask, #1
                        shl     csmask, #1
                        or      outa, csmask                    ' high
                        or      dira, csmask                    ' output

done                    mov     t1, #0
                        wrlong  t1, par                         ' alert foreground

                        
get_cmd                 rdlong  t1, par                 wz      ' check for command
        if_z            jmp     #get_cmd
                                               

shift_bits              and     outa, csmask                    ' cs low
                        mov     bc, #16                         ' set bit count

:loop                   rcl     t1, #1                  wc      ' get t1.31 (MSBFIRST)
                        muxc    outa, dmask                     ' c -> dpin
                        or      outa, cmask                     ' blip clock
                        andn    outa, cmask
                        djnz    bc, #:loop                      ' shift all bits

                        or      outa, csmask                    ' cs high
                        jmp     #done

' --------------------------------------------------------------------------------------------------

t1                      res     1                               ' temp/work vars
t2                      res     1

dmask                   res     1                               ' pin masks
cmask                   res     1
csmask                  res     1

bc                      res     1                               ' bit count for shifting

                        fit     496
                                    

                    
dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      